//-*-C++-*- Header file for the WorkManDB Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _WorkManDB_h_
#define _WorkManDB_h_
#include "AudDB.h"

//
#define TMPDBFILE "/tmp/tmpWMdb"
#define IGNORECOUNT 4


/**
  {\bf WorkManDB} is a {\bf AudDB} implemented to work with
  WorkMan-style databases.

  @memo WorkMan-style audio database.
 */
class WorkManDB : public AudDB
{
   public:
      /** Class Constructor
	   @memo
	  */
	 WorkManDB(void);

      /** Overloaded Constructor.  Sets \Ref{db_name} to
	   a duplicate of the argument {\em fname}.
	   @memo
	  */
	 WorkManDB(char *fname);

      /** Class Destructor.
	   @memo
	   */
	 ~WorkManDB(void);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version. 
	   Because WorkMan-style databases reside in a single file, {\bf
	   open()} also collects and stores the seek-positions of each
	   record in the database.
	   @memo
	   */
	 void open(void);
	 //
	 void dump(void);
		  

   protected:
	 /** 
	   An array containing the position of each record in the file.
	   Has the same length as \Ref{magiclist}.
	   @memo
	   */
	 streampos *seeklist;

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version.  Replaces {\em ct} if
	   it's already in the database.  If it's not, it appends {\em
	   ct} to the database, and increments {\em oldlen}.
	   @memo
	   */
	 void write(Catalog& ct, int& oldlen);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version. Opens an input stream,
	   seeks the position at \Ref{seeklist}[{\em idx}], then calls
	   \Ref{readEntry}.
	   @memo
	   */
	 bool read_i(Catalog& ct, int idx);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version.
	   @memo
	   */
	 bool readEntry(ifstream& instr, Catalog& ct);

	 /** {\tt [virtual]}

	   Reimplements the \Ref{AudDB} version.
	   @memo
	   */
	 bool writeEntry(ofstream& outstr, Catalog& ct);

	 /** 
	   Called by \Ref{open}().  Once \Ref{len_magic} has been set to
	   some nonzero value, {\bf rescanDB} scans the input stream,
	   {\em instr}, for {\em len_magic} records, storing their magic
	   numbers and their seek positions.
	   @memo
	   */
	 void rescanDB(ifstream& instr);

	 /**
	   Appends the \Ref{Catalog} {\em ct} to the end of the
	   database.  Stores the new position of {\em ct} in
	   \Ref{seeklist}, but doesn't update anything else.
	   @memo
	   */
	 bool appendEntry(Catalog& ct);

	 /**
	   Deletes the entry {\em ct} from the database.  It is {\bf {\em
	   very careful}} about doing this.  When done, it decrements
	   \Ref{len_magic} and performs a \Ref{rescanDB} on the altered
	   database file.
	   @memo
	   */
	 bool deleteEntry(Catalog& ct);

	 /**
	   Replaces an exiting entry, {\em ct}, in the database.  Does so
	   by first calling \Ref{deleteEntry}, then calling
	   \Ref{appendEntry}, and lastly updating \Ref{magiclist}, since
	   only \Ref{seeklist} gets updated by \Ref{appendEntry}.
	   @memo
	   */
	 bool replaceEntry(Catalog& ct);
};



// Undocumented test method.
//
inline
void WorkManDB::dump(void)
{
   int i;
   cout << "Len=" << len_magic << endl;
   for(i=0; i<len_magic; i++)
	 {
	 cout << "magic[" << i << "]=" << magiclist[i] 
		 << "\t\tseekpos[" << i << "]=" 
		 << seeklist[i] << endl;
	 }
}





#endif // _WorkManDB_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
